/*********************************************************
 * ParkingManager v. 1.0
 * 
 * The main class deploys the application components 
 * (i.e. contexts, controllers) as well as presence 
 * sensors. We instantiate 100 presence sensors per 
 * parking lot (i.e., A22, B16, D6, Z52). The main class 
 * provides the following scenarios :
 * 
 * 1. Presence sensors publish every 10s the same boolean 
 * value (negated every 10s)
 * 
 * 2. Presence sensors publish every 10s a random boolean 
 * value
 * 
 *********************************************************/

package parkingmanager;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

import parkingmanager.device.CityEntrancePanel;
import parkingmanager.device.Messenger;
import parkingmanager.device.ParkingEntrancePanel;
import parkingmanager.device.PresenceSensor;
import fr.inria.phoenix.diasuite.framework.datatype.parkinglotenum.ParkingLotEnum;

public class Main {

	static List<PresenceSensor> presenceSensorList;

	public static void main(String[] args) {
		System.out.println("[" + getCurrentTime() + "] "
				+ "Starting the parking management system ... ");

		ComponentBinder cb = new ComponentBinder();

		// Deploy all application components (i.e., contexts, controllers)
		cb.deployAll();

		// Deploy all actuators (display panels, messengers, etc.)
		deployActuators(cb);

		// Deploy 100 presence sensors per parking lot ()
		presenceSensorList = deployPresenceSensors(cb);

		System.out
				.println("---------------------------------------------------------------------\n");
		// Publish every 10s a random boolean value
		scenario_2(presenceSensorList);

		// Publish every 10s the same boolean value (negated every 10s)
		// scenario_1(presenceSensorList);
	}

	private static List<PresenceSensor> deployPresenceSensors(ComponentBinder cb) {
		List<PresenceSensor> presenceSensorList = new ArrayList<PresenceSensor>();

		for (ParkingLotEnum plot : ParkingLotEnum.values())
			for (int i = 1; i <= 100; i++) {
				String sensorID = i + "_" + plot.toString();
				presenceSensorList.add(new PresenceSensor(cb
						.getServiceConfiguration("PresenceSensor_" + sensorID),
						sensorID, plot));
			}

		for (PresenceSensor presenceSensor : presenceSensorList)
			cb.deploy(presenceSensor);

		return presenceSensorList;
	}

	private static void deployActuators(ComponentBinder cb) {
		String defaultParkingEntrancePanelID = "pEntrancePanel_1";
		cb.deploy(new ParkingEntrancePanel(cb
				.getServiceConfiguration("ParkingEntrancePanel_"
						+ defaultParkingEntrancePanelID),
				defaultParkingEntrancePanelID, null));

		String defaultCityEntrancePanelID = "cEntrancePanel_1";
		cb.deploy(new CityEntrancePanel(cb
				.getServiceConfiguration("CityEntrancePanel_"
						+ defaultCityEntrancePanelID),
				defaultCityEntrancePanelID, null));

		String defaultMessengerID = "messenger_1";
		cb.deploy(new Messenger(cb.getServiceConfiguration("Messenger_"
				+ defaultMessengerID), defaultMessengerID));
	}

	@SuppressWarnings("unused")
	private static void scenario_1(List<PresenceSensor> presenceSensorList) {
		Boolean value = false;

		while (true) {
			value = !value;
			publishBooleanEveryMillis(presenceSensorList, 10000, value);
		}
	}

	private static void scenario_2(List<PresenceSensor> presenceSensorList) {
		while (true) {
			publishRandomBooleanEveryMillis(presenceSensorList, 10000);
		}
	}

	private static void publishBooleanEveryMillis(
			List<PresenceSensor> presenceSensorList, long millis, Boolean value) {

		try {
			for (PresenceSensor presenceSensor : presenceSensorList) {
				presenceSensor.publish(value);
			}

			Thread.sleep(millis);

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private static void publishRandomBooleanEveryMillis(
			List<PresenceSensor> presenceSensorList, long millis) {

		try {
			for (PresenceSensor presenceSensor : presenceSensorList) {
				presenceSensor.publish(new Random().nextBoolean());
			}

			Thread.sleep(millis);

		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	public static String getCurrentTime() {
		long currentTimeInMillis = System.currentTimeMillis();
		SimpleDateFormat format = new SimpleDateFormat("hh:mm:ss");
		return format.format(new Date(currentTimeInMillis));
	}
}
